@@ -65,30 +65,43 @@ test_that("checkLayout() errors if primary doc & app.R", {
     "myscript.R" = ""
   ))
 
-  expect_snapshot(checkAppLayout(dir, "myscript.R"), error = TRUE)
-})
-
-test_that("checkLayout fails if no known structure", {
-  dir <- local_temp_app(list(
-    "data.txt" = "",
-    "cats.csv" = ""
-  ))
-
-  expect_snapshot(checkAppLayout(dir), error = TRUE)
+  expect_snapshot(checkAppLayout(dir, appPrimaryDoc = "myscript.R"), error = TRUE)
 })
 
 test_that("checkLayout succeeds with some common app structures", {
   rmd <- local_temp_app(list("foo.Rmd" = ""))
   expect_no_error(checkAppLayout(rmd))
 
+  qmd <- local_temp_app(list("foo.qmd" = ""))
+  expect_no_error(checkAppLayout(qmd))
+
+  # perhaps script.R is a pre-render script that generates *.qmd
+  project <- local_temp_app(list("_quarto.yml" = "", "script.R" = ""))
+  expect_no_error(checkAppLayout(project))
+
+  md <- local_temp_app(list("_quarto.yml" = "", "index.md" = ""))
+  expect_no_error(checkAppLayout(md))
+
   shiny1 <- local_temp_app(list("app.R" = ""))
-  expect_no_error(checkAppLayout(rmd))
+  expect_no_error(checkAppLayout(shiny1))
 
   shiny2 <- local_temp_app(list("server.R" = "", "ui.R" = ""))
-  expect_no_error(checkAppLayout(rmd))
+  expect_no_error(checkAppLayout(shiny2))
+
+  api <- local_temp_app(list("plumber.R" = ""))
+  expect_no_error(checkAppLayout(api))
 
   static <- local_temp_app(list("foo.html" = ""))
-  expect_no_error(checkAppLayout(rmd))
+  expect_no_error(checkAppLayout(static))
+
+  staticxml <- local_temp_app(list("data.xml" = ""))
+  expect_no_error(checkAppLayout(staticxml))
+
+  staticdata <- local_temp_app(list(
+    "data.txt" = "",
+    "cats.csv" = ""
+  ))
+  expect_no_error(checkAppLayout(staticdata))
 })
 
 # inferAppMode ------------------------------------------------------------
@@ -103,16 +116,47 @@ test_that("can infer mode for shiny apps", {
   expect_equal(inferAppMode("server.R"), "shiny")
 })
 
-test_that("can infer mode for static quarto and rmd docs", {
+test_that("can infer mode for static rmd", {
   dir <- local_temp_app(list("foo.Rmd" = ""))
   paths <- list.files(dir, full.names = TRUE)
-
   expect_equal(inferAppMode(paths), "rmd-static")
+})
+
+test_that("can infer mode for rmd as static quarto with guidance", {
+  dir <- local_temp_app(list("foo.Rmd" = ""))
+  paths <- list.files(dir, full.names = TRUE)
   expect_equal(inferAppMode(paths, usesQuarto = TRUE), "quarto-static")
+})
+
+test_that("can infer mode for rmd as shiny quarto with guidance", {
   # Static R Markdown treated as rmd-shiny for shinyapps targets
+  dir <- local_temp_app(list("foo.Rmd" = ""))
+  paths <- list.files(dir, full.names = TRUE)
   expect_equal(inferAppMode(paths, isShinyappsServer = TRUE), "rmd-shiny")
 })
 
+test_that("can infer mode for static quarto", {
+  dir <- local_temp_app(list("foo.qmd" = ""))
+  paths <- list.files(dir, full.names = TRUE)
+  expect_equal(inferAppMode(paths), "quarto-static")
+
+  dir <- local_temp_app(list("_quarto.yml" = "", "foo.qmd" = ""))
+  paths <- list.files(dir, full.names = TRUE)
+  expect_equal(inferAppMode(paths), "quarto-static")
+
+  dir <- local_temp_app(list("_quarto.yml" = "", "foo.rmd" = ""))
+  paths <- list.files(dir, full.names = TRUE)
+  expect_equal(inferAppMode(paths), "quarto-static")
+
+  dir <- local_temp_app(list("_quarto.yml" = "", "foo.r" = ""))
+  paths <- list.files(dir, full.names = TRUE)
+  expect_equal(inferAppMode(paths), "quarto-static")
+
+  dir <- local_temp_app(list("foo.r" = ""))
+  paths <- list.files(dir, full.names = TRUE)
+  expect_equal(inferAppMode(paths), "quarto-static")
+})
+
 test_that("can infer mode for shiny rmd docs", {
   yaml_runtime <- function(runtime) {
     c("---", paste0("runtime: ", runtime), "---")
@@ -205,7 +249,10 @@ test_that("otherwise fails back to first file with matching extensions", {
 test_that("errors if no files with needed extension", {
   expect_snapshot(error = TRUE, {
     inferAppPrimaryDoc(NULL, "a.R", "static")
-    inferAppPrimaryDoc(NULL, "a.R", "rmd-shiny")
+    inferAppPrimaryDoc(NULL, "a.html", "rmd-static")
+    inferAppPrimaryDoc(NULL, "a.html", "rmd-shiny")
+    inferAppPrimaryDoc(NULL, "a.html", "quarto-static")
+    inferAppPrimaryDoc(NULL, "a.html", "quarto-shiny")
   })
 })
 
